<?php
/*--------------------------------------------------------------------------------------------------
    Validator.php 2019-10-14
    Gambio GmbH
    http://www.gambio.de
    Copyright (c) 2019 Gambio GmbH
    Released under the GNU General Public License (Version 2)
    [http://www.gnu.org/licenses/gpl-2.0.html]
    --------------------------------------------------------------------------------------------------
 */

namespace Gambio\StyleEdit\Core\Components\Theme;

use Exception;
use FileNotFoundException;
use FilesystemAdapter;
use Gambio\StyleEdit\Core\Components\Theme\Json\ThemeInheritanceHandler;
use Gambio\StyleEdit\Core\SingletonPrototype;
use Gambio\StyleEdit\StyleEditConfiguration;
use stdClass;

/**
 * Class Validator
 *
 * Checks if a theme can be opened in StyleEdit4.
 * 2 files are needed for that.
 *
 * settings.json
 * * * * *  Must be a valid json
 * * * * *  content must be an array
 * * * * *  can be an empty array
 * * * * *  if not empty only objects inside that array
 *
 * theme.json
 * * * * *  Must be a valid json
 * * * * *  content must be an object
 * * * * *  content must have an id
 * * * * *  content must have a thumbnail
 * * * * *  content must have a config object
 * * * * * * * * *  with a basics & areas object
 */
class Validator
{
    /**
     * @var string
     */
    protected $themeName;
    
    /**
     * @var FilesystemAdapter
     */
    protected $filesystemThemes;
    
    /**
     * @var bool
     */
    protected $themeCanBeOpenedInStyleEdit4 = true;
    /**
     * @var StyleEditConfiguration
     */
    protected $styleEditConfiguration;
    
    
    /**
     * @param string                 $themeName
     *
     * @param FilesystemAdapter|null $adapter
     *
     * @return Validator
     * @throws FileNotFoundException
     * @throws Exception
     */
    public static function for(string $themeName, ?FilesystemAdapter $adapter = null): self
    {
        // the SingletonPrototype can only construct this if the inject method of the dependencyInjector class was executed!
        $filesystemThemes = $adapter ?? SingletonPrototype::instance()->get(FilesystemAdapter::class);
        $styleEditConfiguration = SingletonPrototype::instance()->get(StyleEditConfiguration::class);
        
        return new static($themeName, $filesystemThemes, $styleEditConfiguration);
    }
    
    
    /**
     * Validator constructor.
     *
     * @param string                 $themeName
     * @param FilesystemAdapter      $filesystemThemes
     *
     * @param StyleEditConfiguration $styleEditConfiguration
     *
     * @throws FileNotFoundException
     */
    public function __construct(string $themeName, FilesystemAdapter $filesystemThemes, StyleEditConfiguration $styleEditConfiguration)
    {
        $this->themeName        = $themeName;
        $this->filesystemThemes = $filesystemThemes;
        $this->styleEditConfiguration = $styleEditConfiguration;
        
        $this->validateSettingsJson();
        
        if ($this->themeCanBeOpenedInStyleEdit4) {
            
            $this->validateThemeJson();
        }
    }
    
    
    /**
     * @throws FileNotFoundException
     */
    protected function validateSettingsJson(): void
    {
        $settingsJsonPath = $this->themeName . DIRECTORY_SEPARATOR . 'settings.json';
        
        //  every theme needs a settings.json
        if (!$this->filesystemThemes->has($settingsJsonPath)) {
            
            $this->themeCanBeOpenedInStyleEdit4 = false;
            
            return;
        }
        
        $settingsJsonContent = $this->filesystemThemes->read($settingsJsonPath);
        $settingsJson        = json_decode($settingsJsonContent, false);
        
        //  content has to be an array
        if (!is_array($settingsJson)) {
            
            $this->themeCanBeOpenedInStyleEdit4 = false;
            
            return;
        }
        
        //  a settings.json can be empty array
        if (empty($settingsJson)) {
            return;
        }
        
        // everything inside the array must be stdClass
        foreach ($settingsJson as $option) {
            
            if (!is_a($option, stdClass::class)) {
                
                $this->themeCanBeOpenedInStyleEdit4 = false;
            }
        }
    }
    
    
    /**
     *
     * @throws FileNotFoundException
     */
    protected function validateThemeJson(): void
    {
        $themeJsonPath = $this->themeName . DIRECTORY_SEPARATOR . 'theme.json';
        
        if (!$this->filesystemThemes->has($themeJsonPath)) {
            
            $this->themeCanBeOpenedInStyleEdit4 = false;
            
            return;
        }
    
    
        $jsonInheritanceHandler = SingletonPrototype::instance()->get(ThemeInheritanceHandler::class);
        $jsonInheritanceHandler->setFilename($this->styleEditConfiguration->themesFolderPath().$themeJsonPath);
        $themeJson = $jsonInheritanceHandler->execute();
        
        if (!$themeJson instanceof stdClass
            || !isset($themeJson->id, $themeJson->thumbnail, $themeJson->config, $themeJson->config->basics, $themeJson->config->areas)
            || !$themeJson->config->basics instanceof stdClass
            || !$themeJson->config->areas instanceof stdClass) {
            
            $this->themeCanBeOpenedInStyleEdit4 = false;
        }
    }
    
    
    /**
     * @return bool
     */
    public function canBeOpenedInStyleEdit4(): bool
    {
        return $this->themeCanBeOpenedInStyleEdit4;
    }
}