<?php
/* --------------------------------------------------------------
  VariantRepository.php 2019-10-14
  Gambio GmbH
  http://www.gambio.de
  Copyright (c) 2019 Gambio GmbH
  Released under the GNU General Public License (Version 2)
  [http://www.gnu.org/licenses/gpl-2.0.html]
  --------------------------------------------------------------*/

namespace Gambio\StyleEdit\Core\Components\Variant\Repositories;

use Exception;
use Gambio\StyleEdit\Core\Components\Theme\Entities\Interfaces\CurrentThemeInterface;
use Gambio\StyleEdit\Core\Components\Variant\Json\VariantInheritanceHandler;
use Gambio\StyleEdit\Core\Json\FileIO;
use Gambio\StyleEdit\Core\Repositories\Entities\ConfigurationCollection;
use Gambio\StyleEdit\Core\SingletonPrototype;
use Gambio\StyleEdit\StyleEditConfiguration;
use RuntimeException;
use stdClass;

/**
 * Class VariantRepository
 *
 * @package Gambio\StyleEdit\Core\Components\Variant\Repositories
 */
class VariantRepository
{
    /**
     * @var StyleEditConfiguration
     */
    protected $configuration;
    /**
     * @var CurrentThemeInterface
     */
    protected $themeId;
    /**
     * @var FileIO
     */
    protected $fileIO;
    
    
    /**
     * VariantRepository constructor.
     *
     * @param StyleEditConfiguration $configuration
     * @param FileIO                 $fileIO
     * @param CurrentThemeInterface  $currentTheme
     */
    public function __construct(
        StyleEditConfiguration $configuration,
        FileIO $fileIO,
        CurrentThemeInterface $currentTheme
    ) {
        $this->configuration = $configuration;
        $this->fileIO        = $fileIO;
        $this->themeId       = $currentTheme->id();
    }
    
    
    /**
     * @param string $themeId
     *
     * @return VariantRepository
     * @throws Exception
     */
    public static function createForTheme(string $themeId)
    {
        /**
         * @var VariantRepository $result
         */
        $result = SingletonPrototype::instance()->get(VariantRepository::class);
        $result->setThemeId($themeId);
        
        return $result;
    }
    
    
    /**
     * @param string $themeId
     */
    protected function setThemeId(string $themeId): void
    {
        $this->themeId = $themeId;
    }
    
    
    /**
     * @param string $directory relative path to variant option
     *
     * @return stdClass
     *
     * @throws Exception
     */
    public function loadVariantJson(string $directory): stdClass
    {
        $variantFilename  = $this->getVariantFilename($directory);
        $settingsFilename = dirname($variantFilename) . DIRECTORY_SEPARATOR . 'settings.json';
        
        if (!$this->fileIO->exists($variantFilename)) {
            throw new RuntimeException($variantFilename . ' does not exist');
        }
        
        /**
         * @var VariantInheritanceHandler $handler
         */
        $handler = SingletonPrototype::instance()->get(VariantInheritanceHandler::class);
        $handler->setFilename($variantFilename);
        $variantJson = $handler->execute();
        
        if ($this->fileIO->exists($settingsFilename)) {
            /**
             * @var VariantConfigurationRepository $configurationRepository
             */
            
            $configurationRepository = SingletonPrototype::instance()
                ->get(VariantConfigurationRepository::class, $settingsFilename);
            $this->initializeValuesFromVariantSettingsJson($variantJson,
                                                           $configurationRepository->configurationsList());
        }
        
        return $variantJson;
    }
    
    
    /**
     * @param $directory
     *
     * @return string
     */
    protected function getVariantFilename($directory): string
    {
        return implode(DIRECTORY_SEPARATOR,
                       [
                           $this->configuration->themesFolderPath(),
                           $this->themeId,
                           $directory,
                           'variant.json'
                       ]);
    }
    
    
    /**
     * @param stdClass                $categories
     * @param ConfigurationCollection $configurationCollection
     *
     * @throws Exception
     */
    protected function initializeValuesFromVariantSettingsJson(
        stdClass $categories,
        ConfigurationCollection $configurationCollection
    ): void {
        if (isset($categories->categories)) {
            foreach ($categories->categories as $category) {
                $this->initializeValuesFromVariantSettingsJson($category, $configurationCollection);
            }
        } elseif (isset($categories->fieldsets)) {
            $this->initializeValueInFieldSets($categories->fieldsets, $configurationCollection);
        }
    }
    
    
    /**
     * @param array                   $fieldSets
     * @param ConfigurationCollection $configurationCollection
     */
    protected function initializeValueInFieldSets(
        array $fieldSets,
        ConfigurationCollection $configurationCollection
    ): void {
        foreach ($fieldSets as $fieldset) {
            foreach ($fieldset->options as $option) {
                if ($configurationCollection && $configurationCollection->keyExists($option->id)) {
                    $option->value = $configurationCollection->getValue($option->id)->value();
                } else {
                    $option->value = $option->default;
                }
            }
        }
    }
    
    
    /**
     * @param string $directory relative path to variant option
     *
     * @return bool
     * @throws Exception
     */
    public function variantJsonExists(string $directory): bool
    {
        return $this->fileIO->exists($this->getVariantFilename($directory));
    }
}
