<?php
/* --------------------------------------------------------------
   VariantService.php 2019-10-24
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2019 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

namespace Gambio\StyleEdit\Core\Components\Variant\Services;

use Exception;
use Gambio\StyleEdit\Core\Components\Theme\Entities\Interfaces\CurrentThemeInterface;
use Gambio\StyleEdit\Core\Components\Variant\Repositories\VariantRepository;
use Gambio\StyleEdit\Core\Language\Services\LanguageService;
use Gambio\StyleEdit\Core\SingletonPrototype;
use Gambio\StyleEdit\StyleEditConfiguration;
use ReflectionException;
use stdClass;

/**
 * Class VariantService
 * @package Gambio\StyleEdit\Core\Components\Variant\Services
 */
class VariantService
{
    
    const DEFAULT_THUMBNAIL_FILENAME = 'thumbnail.png';
    /**
     * @var LanguageService
     */
    protected $languageService;
    /**
     * @var VariantRepository
     */
    protected $repository;
    /**
     * @var StyleEditConfiguration
     */
    protected $settings;
    /**
     * @var string
     */
    protected $themeId;
    
    
    /**
     * VariantService constructor.
     *
     * @param StyleEditConfiguration $settings
     * @param LanguageService        $languageService
     * @param VariantRepository      $repository
     * @param CurrentThemeInterface  $currentTheme
     *
     * @throws Exception
     */
    public function __construct(
        StyleEditConfiguration $settings,
        LanguageService $languageService,
        VariantRepository $repository,
        CurrentThemeInterface $currentTheme = null
    ) {
        $this->settings        = $settings;
        $this->languageService = $languageService;
        $this->repository      = $repository;
        
        if ($currentTheme) {
            $this->themeId = $currentTheme->id();
        }
    }
    
    
    /**
     * @param $themeId
     *
     * @return VariantService
     * @throws Exception
     */
    public static function createForTheme($themeId): VariantService
    {
        /**
         * @var VariantService $result
         */
        $result = SingletonPrototype::instance()->get(VariantService::class);
        $result->setThemeId($themeId);
        return $result;
    }
    
    
    /**
     * @param string $themeId
     *
     * @throws Exception
     */
    protected function setThemeId(string $themeId): void
    {
        $this->themeId = $themeId;
        $this->repository = VariantRepository::createForTheme($this->themeId);
    }
    
    
    /**
     * @param array $category
     *
     * @return array $category with variant categories
     *
     * @throws ReflectionException
     * @throws Exception
     */
    public function addVariantCategoriesToCategory(array $category): array
    {
        $variantCategories = [];
        
        foreach ($category as &$categories) {
            
            foreach ($categories->fieldsets as &$fieldset) {
                
                foreach ($fieldset->options as &$options) {
                    
                    if ($options->type === 'variant') {
                        
                        foreach ($options->options as &$option) {
                            
                            $option->dir = 'variants/' . $options->id . '/' . $option->id;
                            // add variant's thumbnail
                            $option->thumbnail = $this->getThumbnailPath($option->dir) . ($option->thumbnail ??
                                                                                          self::DEFAULT_THUMBNAIL_FILENAME);
                            
                            if ($this->repository->variantJsonExists($option->dir)) {
                                
                                $variantJson           = $this->repository->loadVariantJson($option->dir);
                                $variantIdentification = $options->id . '/' . $option->id;
                                
                                $this->setForProperties($variantJson, $variantIdentification);
                                $this->translateVariantProperties($variantJson);
                                
                                $variantCategories[] = $variantJson;
                            }
                        }
                    }
                }
            }
        }
        
        unset($categories, $fieldset, $options, $option);
        
        array_splice($category, 1, 0, $variantCategories);
        
        return $category;
    }
    
    
    /**
     * @param string $variantDir
     *
     * @return string
     * @throws Exception
     */
    protected function getThumbnailPath(string $variantDir): string
    {
        return (ENABLE_SSL ? HTTPS_SERVER : HTTP_SERVER) . DIR_WS_CATALOG . "themes/{$this->themeId}/{$variantDir}/";
    }
    
    
    /**
     * @param stdClass $variantJson
     * @param string   $variantIdentification
     */
    protected function setForProperties(stdClass $variantJson, string $variantIdentification): void
    {
        $variantJson->for = $variantIdentification;
        
        if (isset($variantJson->categories)) {
            
            foreach ($variantJson->categories as $category) {
                
                $this->setForProperties($category, $variantIdentification);
            }
        }
        
        if (isset($variantJson->fieldsets)) {
            
            foreach ($variantJson->fieldsets as $fieldset) {
                
                $fieldset->for = $variantIdentification;
                
                if (isset($fieldset->options)) {
                    
                    foreach ($fieldset->options as $option) {
                        
                        $option->for = $variantIdentification;
                    }
                }
            }
        }
    }
    
    
    /**
     * @param stdClass $variantJson
     *
     * @throws ReflectionException
     * @throws Exception
     */
    protected function translateVariantProperties(stdClass $variantJson): void
    {
        /**
         * @var LanguageService $languageService
         */
        $languageService = $this->languageService();
        
        $variantJson->title = $languageService->translate($variantJson->title);
        
        if (isset($variantJson->categories)) {
            
            foreach ($variantJson->categories as $category) {
                
                $this->translateVariantProperties($category);
            }
        }
        
        if (isset($variantJson->fieldsets)) {
            
            foreach ($variantJson->fieldsets as $fieldset) {
                
                $fieldset->title = $languageService->translate($fieldset->title);
                
                foreach ($fieldset->options as $option) {
                    
                    $option->label = $languageService->translate($option->label);
                }
            }
        }
    }
    
    
    /**
     * @return LanguageService
     * @throws Exception
     */
    protected function languageService(): LanguageService
    {
        if ($this->languageService === null) {
            
            $this->languageService = SingletonPrototype::instance()->get(LanguageService::class);
        }
        
        return $this->languageService;
    }
    
    
    /**
     * @return VariantRepository
     */
    protected function respository(): VariantRepository
    {
        return $this->repository;
    }
}